/*
 * Copyright 2013 Mariusz Kopylec
 *
 * This file is part of SQL Mapper.
 *
 * SQL Mapper is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * SQL Mapper is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Foobar. If not, see http://www.gnu.org/licenses/
 */
package net.sourceforge.kopylec.sqlmapper.core;

import java.sql.Connection;

/**
 * A transaction isolation level which represents {@code java.sql.Connection}
 * transaction constants.
 *
 * @author Mariusz Kopylec
 * @version 1.0
 * @see Connection
 */
public enum TransactionLevel {

    /**
     * {@link Connection#TRANSACTION_NONE}
     */
    NONE(Connection.TRANSACTION_NONE),
    /**
     * {@link Connection#TRANSACTION_READ_COMMITTED}
     */
    READ_COMMITTED(Connection.TRANSACTION_READ_COMMITTED),
    /**
     * {@link Connection#TRANSACTION_READ_UNCOMMITTED}
     */
    READ_UNCOMMITTED(Connection.TRANSACTION_READ_UNCOMMITTED),
    /**
     * {@link Connection#TRANSACTION_REPEATABLE_READ}
     */
    REPEATABLE_READ(Connection.TRANSACTION_REPEATABLE_READ),
    /**
     * {@link Connection#TRANSACTION_SERIALIZABLE}
     */
    SERIALIZABLE(Connection.TRANSACTION_SERIALIZABLE);
    private final int level;

    private TransactionLevel(int level) {
        this.level = level;
    }

    int getLevel() {
        return level;
    }
}
