package net.sourceforge.kopylec;

import java.util.List;
import net.sourceforge.kopylec.dao.CountryDAO;
import net.sourceforge.kopylec.entity.City;
import net.sourceforge.kopylec.entity.Country;
import net.sourceforge.kopylec.sqlmapper.exception.SQLMapperException;
import org.postgresql.ds.PGSimpleDataSource;

/**
 * A demostration of
 * <code>sql-mapper</code>.
 *
 * @author Mariusz Kopylec
 */
public class Application {

    private final CountryDAO dao;

    public static void main(String[] args) {
        Application application = new Application();
        application.createPoland();
        application.createCanada();
        application.printPoland();
        application.printAllCountries();
        application.printTorontoPopulation();
        application.printAllCitiesPopulations();
        application.deleteAllCountriesData();
    }

    public Application() {
        PGSimpleDataSource dataSource = new PGSimpleDataSource();
        dataSource.setDatabaseName("countries");
        dataSource.setServerName("localhost");
        dataSource.setPortNumber(5432);
        dataSource.setUser("postgres");
        dataSource.setPassword("admin");

        dao = new CountryDAO(dataSource);
    }

    public void createPoland() {
        System.out.println("\nCreating Poland...");
        try {
            dao.openSession();

            dao.insertCountry("Poland");
            dao.insertCapital("Warsaw", "Poland");
            dao.insertCity("Kraków", 759137, "Poland");
            dao.insertCity("Łódź", 737098, "Poland");
            dao.insertCity("Wrocław", 631235, "Poland");
        } catch (SQLMapperException ex) {
            // SQLMapperException is a runtime exception so you can catch it or leave it to your global exception handler
            ex.printStackTrace(System.err);
        } finally {
            dao.closeSession();
        }
    }

    public void createCanada() {
        System.out.println("\nCreating Canada...");
        try {
            dao.openSession();

            dao.insertCountry("Canada");
            // Do not create a capital for Canada
            dao.insertCity("Toronto", 5583064, "Canada");
            dao.insertCity("Montreal", 3824221, "Canada");
            dao.insertCity("Vancouver", 2313328, "Canada");
        } finally {
            dao.closeSession();
        }
    }

    public void printPoland() {
        System.out.println("\nPrinting Poland data...");
        try {
            dao.openSession();

            Country poland = dao.selectCountryData("Poland");

            System.out.println("---\nCountry: " + poland);
            if (poland != null) {
                System.out.println("Capital: " + poland.getCapital());
                System.out.print("Cities: ");
                for (City city : poland.getCities()) {
                    System.out.print(city + " ");
                }
                System.out.println();
            }
        } finally {
            dao.closeSession();
        }
    }

    public void printAllCountries() {
        System.out.println("\nPrinting all countries data...");
        try {
            dao.openSession();
            List<Country> countries = dao.selectCountriesData();

            for (Country country : countries) {
                System.out.println("---\nCountry: " + country);
                System.out.println("Capital: " + country.getCapital());
                System.out.print("Cities: ");
                for (City city : country.getCities()) {
                    System.out.print(city + " ");
                }
                System.out.println();
            }
        } finally {
            dao.closeSession();
        }
    }

    public void printTorontoPopulation() {
        System.out.println("\nPrinting Toronto population...");
        try {
            dao.openSession();

            Integer population = dao.selectCityPopulation("Toronto");

            System.out.println("---\nPopulation: " + population);
        } finally {
            dao.closeSession();
        }
    }

    public void printAllCitiesPopulations() {
        System.out.println("\nPrinting all cities populations...");
        try {
            dao.openSession();

            List<Integer> populations = dao.selectCitiesPopulations();

            for (Integer population : populations) {
                System.out.println("---\nPopulation: " + population);
            }
        } finally {
            dao.closeSession();
        }
    }

    public void deleteAllCountriesData() {
        System.out.println("\nDeleting all countries data...");
        try {
            dao.openSession();

            dao.deleteCountries();
        } finally {
            dao.closeSession();
        }
    }
}
