package net.sourceforge.kopylec.dao;

import java.sql.SQLException;
import java.util.List;
import javax.sql.DataSource;
import net.sourceforge.kopylec.entity.Capital;
import net.sourceforge.kopylec.entity.City;
import net.sourceforge.kopylec.entity.Country;
import net.sourceforge.kopylec.sqlmapper.core.MultipleResultsMapper;
import net.sourceforge.kopylec.sqlmapper.core.SingleResultMapper;
import net.sourceforge.kopylec.sqlmapper.dao.DataAccessObject;

public class CountryDAO extends DataAccessObject {

    private static final String INSERT_COUNTRY = "insert into country values (default, ?)";
    private static final String INSERT_CAPITAL = "insert into capital values (default, ?, (select id from country where name = ?))";
    private static final String INSERT_CITY = "insert into city values (default, ?, ?, (select id from country where name = ?))";
    private static final String SELECT_COUNTRY_DATA = "select cn.id as country_id, cn.name as country_name, cp.id as capital_id, "
            + "cp.name as capital_name, ct.id as city_id, ct.name as city_name, ct.population "
            + "from country cn left join capital cp on cn.id = cp.country left join city ct on cn.id = ct.country where cn.name = ?";
    private static final String SELECT_COUNTRIES_DATA = "select cn.id as country_id, cn.name as country_name, cp.id as capital_id, "
            + "cp.name as capital_name, ct.id as city_id, ct.name as city_name, ct.population "
            + "from country cn left join capital cp on cn.id = cp.country left join city ct on cn.id = ct.country";
    private static final String SELECT_CITY_POPULATION = "select population from city where name = ?";
    private static final String SELECT_CITIES_POPULATIONS = "select population from city";
    private static final String DELETE_COUNTRIES = "delete from country";

    public CountryDAO(DataSource dataSource) {
        super(dataSource);
    }

    public void insertCountry(String name) {
        getSqlRunner().update(INSERT_COUNTRY, name);
    }

    public void insertCapital(String name, String countryName) {
        getSqlRunner().update(INSERT_CAPITAL, name, countryName);
    }

    public void insertCity(String name, int population, String countryName) {
        getSqlRunner().update(INSERT_CITY, name, population, countryName);
    }

    public Country selectCountryData(String name) {
        Country country = getSqlRunner().selectOne(SELECT_COUNTRY_DATA, new SingleResultMapper<Country>() {
            @Override
            protected void mapRow() throws SQLException {
                Country country = new Country(getInteger("country_id"), getString("country_name"));
                country = mergeWithResults(country);

                if (isNotNull("capital_id")) {
                    Capital capital = new Capital(getInteger("capital_id"), getString("capital_name"));
                    capital.setCountry(country);
                    country.setCapital(capital);
                }

                City city = new City(getInteger("city_id"), getString("city_name"), getInteger("population"));
                city = merge(city, country.getCities());
                city.setCountry(country);
            }
        }, name);

        return country;
    }

    public List<Country> selectCountriesData() {
        List<Country> countries = getSqlRunner().selectMany(SELECT_COUNTRIES_DATA, new MultipleResultsMapper<Country>() {
            @Override
            protected void mapRow() throws SQLException {
                Country country = new Country(getInteger("country_id"), getString("country_name"));
                country = mergeWithResults(country);

                if (isNotNull("capital_id")) {
                    Capital capital = new Capital(getInteger("capital_id"), getString("capital_name"));
                    capital.setCountry(country);
                    country.setCapital(capital);
                }

                City city = new City(getInteger("city_id"), getString("city_name"), getInteger("population"));
                city = merge(city, country.getCities());
                city.setCountry(country);
            }
        });

        return countries;
    }

    public Integer selectCityPopulation(String name) {
        return getSqlRunner().selectOne(SELECT_CITY_POPULATION, Integer.class, name);
    }

    public List<Integer> selectCitiesPopulations() {
        return getSqlRunner().selectMany(SELECT_CITIES_POPULATIONS, Integer.class);
    }

    public void deleteCountries() {
        getSqlRunner().update(DELETE_COUNTRIES);
    }
}
